//+------------------------------------------------------------------+
//|                                                    JK_MACD_2.mq4 |
//|                Copyright  2006, Jure Kren (jure.kren@gmail.com) |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, Jure Kren (jure.kren@gmail.com)"
#property link      ""

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 DeepSkyBlue
#property indicator_width1 5

#include <JK_Lib.mqh>

extern int    xMAPeriod1    = 1;
extern int    xMAPeriod2    = 144;
extern int    xMAMethod1    = MODE_EMA;
extern int    xMAMethod2    = MODE_EMA;
extern int    xMAPrice1     = PRICE_CLOSE;
extern int    xMAPrice2     = PRICE_CLOSE;
extern int    xDiffOrder    = 0;              // differencing level (1 is good for displaying the change of the diff. between the two EMAs, 0 has no effect)
extern bool   xShowDistrib  = true;
extern string xLevels       = "-600,-550,-500,-450,-400,-350,-300,-250,-200,-150,-100,-50,0,50,100,150,200,250,300,350,400,450,500,550,600";
extern int    xStatBars     = -1;             // number of previous bars to use for calculating the distribution 
extern int    xMaxBarSize   = 50;
extern string xBarString    = "]";

double        gY[];
int           gLevel[];

int init() {
  SetIndexStyle(0, DRAW_HISTOGRAM, STYLE_SOLID);
  SetIndexBuffer(0, gY);
  //
  StrToArrayI(xLevels, ",", gLevel);
  ArraySort(gLevel);
  return(0);
}

int deinit() {
  if (xShowDistrib) {
    Comment("");
  }
  return(0);
}

int start() {
  ArrayMACD(gY, xMAPeriod1, xMAMethod1, xMAPrice1, xMAPeriod2, xMAMethod2, xMAPrice2);
  ArrayMultiply(gY, gY, 1/Point);
  ArrayNthOrderDiff(gY, gY, xDiffOrder);
  if (xShowDistrib) ShowDistrib(gY, gLevel);
  return(0);
}

void ShowDistrib(double& y[], int& level[]) {
  int i, j;
  int num_bars = CondI(xStatBars < 1, ArraySize(y), xStatBars); 
  int num_hits = 0;
  int hits[];
  ArrayResize(hits, ArraySize(level) - 1);
  ArrayInitialize(hits, 0.0);
  
  for (i = 0; i < num_bars - 1; i++) {
    for (j = 0; j < ArraySize(hits); j++) {
      double a = level[j];
      double b = level[j+1];
      if (y[i] >= a && y[i] < b) {
        hits[j]++;
        num_hits++;
      }
    }
  }

  int max_hits = -1;
  for (j = 0; j < ArraySize(hits); j++) {
    if (hits[j] > max_hits) {
      max_hits = hits[j];
    }
  }

  string c = "";  
  int max_bar_size = MathMin(MathMax(1, xMaxBarSize), 100);
  double max_p = max_hits/(1.0*num_hits);
  for (i = 0; i < ArraySize(hits); i++) {
    j = (ArraySize(hits) - 1) - i;
    double p = hits[j]/(1.0*num_hits);
    int bar_size = MathCeil((p/max_p)*max_bar_size);
    string str_bar = StrDup(xBarString, bar_size) + StrDup(" ", (max_bar_size - bar_size));
    c = c + str_bar + "    " + DoubleToStr(100*p, 1) + "% (" + hits[j] + ")   [" + level[j] + ", " + level[j+1] + ") " + "\n";
  }
  Comment(c);
}


